import { Construct } from "constructs";
import { IConnectable, IVpc } from "aws-cdk-lib/aws-ec2";
import { Credentials, DatabaseCluster, IClusterInstance } from "aws-cdk-lib/aws-rds";
import { Cluster } from "aws-cdk-lib/aws-ecs";
import { RemovalPolicy } from "aws-cdk-lib/core";
import type { ISecret } from "aws-cdk-lib/aws-secretsmanager";
export interface DagsterServiceProps {
    /**
     * The VPC to deploy the service to.
     *
     * You must specify either {@link vpc} or {@link cluster}.
     */
    readonly vpc?: IVpc;
    /**
     * The ECS cluster to deploy the service to.
     *
     * You must specify either {@link vpc} or {@link cluster}.
     */
    readonly cluster?: Cluster;
    /**
     * The database to deploy to.
     */
    readonly database?: DagsterDatabaseProps;
    /**
     * The removal policy to use for the database and service.
     *
     * @default - The database is not removed automatically.
     */
    readonly removalPolicy?: RemovalPolicy;
}
export interface DagsterDatabaseProps {
    /**
     * Credentials for the administrative user
     *
     * @default - A username of 'admin' and SecretsManager-generated password
     */
    readonly credentials?: Credentials;
    /**
     * An optional identifier for the cluster
     *
     * @default - A name is automatically generated.
     */
    readonly clusterIdentifier?: string;
    /**
     * The writer instance to use for the database.
     *
     * @default - A serverless instance is created.
     */
    readonly writer?: IClusterInstance;
    /**
     * The readers instances to use for the database.
     *
     * @default - No readers are created.
     */
    readonly readers?: IClusterInstance[];
    /**
     * The port to connect to the database on.
     *
     * @default - 5432
     */
    readonly port?: number;
}
/**
 * Represents a Dagster service deployment in AWS, encapsulating the necessary AWS resources.
 *
 * This class allows for the easy setup of a Dagster service with a connected Aurora Postgres database
 * within an ECS cluster. It abstracts away the complexity of directly dealing with AWS CDK constructs
 * for creating and configuring the ECS service, database, and necessary permissions.
 */
export declare class DagsterService extends Construct {
    readonly database: DatabaseCluster;
    readonly databaseSecret: ISecret;
    constructor(scope: Construct, id: string, props: DagsterServiceProps);
    /**
     * Allow a connectable to access the database.
     *
     * @param connectable The connectable to allow access from.
     */
    allowDBAccessFrom(connectable: IConnectable): void;
}
